/*
 This file is part of the digitalSTROM ds485-core library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM ds485-core library is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation version 2
 of the License.

 digitalSTROM ds485-core library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM ds485-core library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef DS485CONSTV1_H_
#define DS485CONSTV1_H_

#include <time.h>

/*
 * Message Declarations
 */
enum DS485MessageSizes {
    DS485_MIN_DATA_PAYLOAD_SIZE = 1,        /* message ID */
    DS485_HEADER_CRC_SIZE       = 6,        /* start + destination + source + command/length + crc16 */
    DS485_SHORT_PAYLOAD_SIZE    = 15,
    DS485_LONG_PAYLOAD_SIZE     = 128,
    DS485_SHORT_MESSAGE_SIZE    = (DS485_SHORT_PAYLOAD_SIZE+DS485_HEADER_CRC_SIZE),
    DS485_LONG_MESSAGE_SIZE     = (DS485_LONG_PAYLOAD_SIZE+DS485_HEADER_CRC_SIZE)
};

/*
 * dS485 Frame
 */
typedef struct _dS485Frame {
    uint8_t      size;
    uint8_t      data[DS485_SHORT_MESSAGE_SIZE];
} dS485Frame;

/**
 * dS485 Protocol Header
 */
typedef struct _dS485Header {
    unsigned char start;

    unsigned char typeflag : 1;
    unsigned char broadcastflag : 1;
    unsigned char destination : 6;

    unsigned char msgid : 2;
    unsigned char source : 6;
} __attribute__ ((packed)) dS485Header;

/**
 * dS485 Frame
 */
typedef struct _dS485FrameNative {
    unsigned char start;

    unsigned char typeflag : 1;
    unsigned char broadcastflag : 1;
    unsigned char destination : 6;

    unsigned char msgid : 2;
    unsigned char source : 6;

    unsigned char length : 4;
    unsigned char command : 4;

    unsigned char data[15];   /* 15 data bytes max on the wire */

    unsigned short crc16;
} __attribute__ ((packed)) dS485FrameNative;

/**
 * dS485 Datagramm
 */
typedef struct _dS485Packet {
    dS485Frame frame;
    time_t tstamp;
    unsigned short length;
    unsigned short flags;
    unsigned short valid;
    struct _dS485Packet* next;
} dS485Packet;

enum DS485SpecialCharacters {
    DS485_START_BYTE    = 0xFD,
    DS485_ESCAPE_BYTE   = 0xFC,
    DS485_RESERVED_BYTE = 0xFE
};

enum DS485BitMasks {
    DS485_TYPEFLAG_MASK     = 0x01,
    DS485_BROADCAST_MASK    = 0x02,
    DS485_COMMAND_MASK      = 0xF0,
    DS485_LENGTH_MASK       = 0x0F
};

enum DS485SpecialDestination {
    DS485_BROADCAST = 0xFF
};

enum DS485MessageOffset {
    DS485_START_OFFSET  = 0,
    DS485_DEST_OFFSET   = 1,
    DS485_SRC_OFFSET    = 2,
    DS485_CL_OFFSET     = 3,
    DS485_DATA_OFFSET   = 4
};

enum DS485Addresses {
    DS485_MASTER_ADDRESS        = 0,
    DS485_MIN_ADDRESS           = 0,
    DS485_MIN_SLAVE_ADDRESS     = 1,
    DS485_MAX_SLAVE_ADDRESS     = 62,
    DS485_INTERMEDIATE_ADDRESS  = 63,
    DS485_INVALID_ADDRESS       = 255
};

typedef enum {
    DS485_SOLICIT_SUCCESSOR_REQUEST_LONG  = 0,
    DS485_SOLICIT_SUCCESSOR_REQUEST_SHORT = 1,
    DS485_SOLICIT_SUCCESSOR_RESPONSE      = 2,
    DS485_GET_ADDRESS_REQUEST             = 3,
    DS485_GET_ADDRESS_RESPONSE            = 4,
    DS485_SET_DEVICE_ADDRESS_REQUEST      = 5,
    DS485_SET_DEVICE_ADDRESS_RESPONSE     = 6,
    DS485_SET_SUCCESSOR_ADDRESS_REQUEST   = 7,
    DS485_SET_SUCCESSOR_ADDRESS_RESPONSE  = 8,
    DS485_REQUEST                         = 9,
    DS485_RESPONSE                        = 10,
    DS485_ACKNOWLEDGE                     = 11,
    DS485_BUSY                            = 12,
    DS485_DATA                            = 13,
    DS485_PING                            = 14,
    DS485_EVENT                           = 15
} DS485CommandId;

typedef enum {
    DS485_TYPE_REQ                      = DS485_REQUEST,
    DS485_TYPE_ACK                      = DS485_ACKNOWLEDGE,
    DS485_TYPE_BUSY                     = DS485_BUSY,
    DS485_TYPE_RES                      = DS485_RESPONSE,
    DS485_TYPE_EVT                      = DS485_EVENT
} DS485CommandType;


#endif /* DS485CONSTV1_H_ */
